"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class MagicSquareSolver {
    constructor(ms) {
        this.ms = ms;
        try {
            if (!!!ms.square) {
                throw new Error("Le carré magique n'est pas défini");
            }
            if (!!!ms.dimension) {
                throw new Error("La dimension du carré magique n'est pas définie");
            }
            if (ms.dimension != ms.square.length) {
                throw new Error("La dimension renseignée n'est pas en accord avec la dimension diu carré magique");
            }
            for (var i = 0, j = ms.square.length; i < j; i++) {
                if (ms.square[i].length != ms.dimension) {
                    throw new Error("La dimension renseignée n'est pas en accord avec la dimension diu carré magique");
                }
            }
            this.ms.dimension = ms.dimension;
            this.ms.square = ms.square;
            let dimension = (this.ms.dimension * this.ms.dimension);
            this.expected = ((dimension * (dimension + 1)) / 2) / this.ms.dimension;
        }
        catch (error) {
            console.error(error);
        }
    }
    /**
     * Permet de vérifier s'il s'agit bien d'un carré magique
     */
    check() {
        try {
            if (this.ms === undefined) {
                throw new Error("MagicSqaure is undefined");
            }
            let response = { message: null, status: false };
            if (!this.diagonaleLeftToRight()) {
                response.message = "La somme de la diagonale de la gauche vers la droite n'est pas correcte";
                return response;
            }
            if (!this.diagonaleRightToLeft()) {
                response.message = "La somme de la diagonale de la droite vers la gauche n'est pas correcte";
                return response;
            }
            let resultColumnAndLine = this.columnAndline();
            if (!resultColumnAndLine.status) {
                return resultColumnAndLine;
            }
            return response;
        }
        catch (error) {
            console.error(error);
        }
    }
    /**
     * Vérification de la somme des colonnes et des lignes
     * @param j
     */
    columnAndline() {
        var sc = 0;
        var sl = 0;
        for (var i = 0; i < this.ms.dimension; i++) {
            for (var j = 0; j < this.ms.dimension; j++) {
                sl += this.ms.square[i][j];
                sc += this.ms.square[j][i];
            }
            if (sc != this.expected) {
                return {
                    message: "La somme de la colonne " + (i + 1) + " n'est pas bonne (somme = " + sc + "). La somme de la colonne attendue doit être la suivante : " + this.expected,
                    status: false
                };
            }
            if (sl != this.expected) {
                return {
                    message: "Le somme de la ligne " + (i + 1) + " n'est pas bonne (somme = " + sc + "). La somme de la ligne attendue doit être la suivante : " + this.expected,
                    status: false
                };
            }
            sc = 0;
            sl = 0;
        }
        return { message: null, status: true };
    }
    /** Vérification de la somme de la première diagonale (de gauche à droite) */
    diagonaleLeftToRight() {
        var s = 0;
        for (var i = 0; i < this.ms.dimension; i++) {
            s += this.ms.square[i][i];
        }
        return (s == this.expected);
    }
    /** Vérification de la somme de la deuxième diagonale (de droite à gauche) */
    diagonaleRightToLeft() {
        var s = 0;
        for (var i = 0; i < this.ms.dimension; i++) {
            s += this.ms.square[i][this.ms.dimension - i - 1];
        }
        return (s == this.expected);
    }
}
exports.MagicSquareSolver = MagicSquareSolver;
